@@ -1,5 +1,6 @@
 import sys
-from typing import List, Sequence, TextIO, Union
+from collections.abc import Sequence
+from typing import TextIO
 
 from rich.console import Console
 from rich.table import Table
@@ -34,7 +35,7 @@ POS_MARK = {
 
 
 def draw_tree(
-    leaves: Union[Sequence[Phrase], Sequence[BasePhrase]],
+    leaves: Sequence[Phrase] | Sequence[BasePhrase],
     fh: TextIO = sys.stdout,
     show_pos: bool = False,
     show_rel: bool = False,
@@ -72,20 +73,19 @@ def draw_tree(
             elif j == parent_index:
                 if dep_type in (DepType.PARALLEL, DepType.IMPERFECT_PARALLEL, DepType.APPOSITION):
                     item[i][j] = str(dep_type.value)
+                elif active_column[j] == 2:
+                    item[i][j] = "┨"
+                elif active_column[j] == 1:
+                    item[i][j] = "┤"
                 else:
-                    if active_column[j] == 2:
-                        item[i][j] = "┨"
-                    elif active_column[j] == 1:
-                        item[i][j] = "┤"
-                    else:
-                        item[i][j] = "┐"
+                    item[i][j] = "┐"
                 if active_column[j] == 2:
                     pass
                 elif para_row:
                     active_column[j] = 2
                 else:
                     active_column[j] = 1
-            else:
+            else:  # noqa: PLR5501
                 if active_column[j] == 2:
                     item[i][j] = "┃"
                 elif active_column[j] == 1:
@@ -93,7 +93,7 @@ def draw_tree(
                 else:
                     item[i][j] = " "
 
-    lines: List[str] = []
+    lines: list[str] = []
     for i in range(len(leaves)):
         line = _leaf_string(leaves[i], show_pos)
         for j in range(i + 1, len(leaves)):
@@ -101,7 +101,7 @@ def draw_tree(
         lines.append(line)
 
     max_length = max(_str_real_length(line) for line in lines)
-    for line, leaf in zip(lines, leaves):
+    for line, leaf in zip(lines, leaves, strict=True):
         diff = max_length - _str_real_length(line)
         tree_string = " " * diff + line
         feat_string = _feat_string(leaf, show_rel, show_pas) if isinstance(leaf, BasePhrase) else ""
@@ -118,7 +118,7 @@ def _extend_horizontal(token: str) -> str:
         return " "
 
 
-def _leaf_string(leaf: Union[Phrase, BasePhrase], show_pos: bool) -> str:
+def _leaf_string(leaf: Phrase | BasePhrase, show_pos: bool) -> str:
     ret = ""
     for morpheme in leaf.morphemes:
         ret += morpheme.text
@@ -135,7 +135,7 @@ def _str_real_length(string: str) -> int:
 
 
 def _feat_string(base_phrase: BasePhrase, show_rel: bool, show_pas: bool) -> str:
-    tag_strings: List[str] = []
+    tag_strings: list[str] = []
     if show_rel is True:
         for tag in base_phrase.rel_tags:
             tag_strings.append(f"{tag.type}:{tag.target}")
@@ -153,13 +153,13 @@ def _get_core_text(base_phrase: BasePhrase) -> str:
     """Get the core text without ancillary words."""
     morphemes = base_phrase.morphemes
     start_index = 0
-    for i, morpheme in enumerate(morphemes):
+    for morpheme in morphemes:
         if morpheme.pos in ("助詞", "特殊", "判定詞"):
             start_index += 1
         else:
             break
     end_index = len(morphemes)
-    for i, morpheme in enumerate(reversed(morphemes)):
+    for morpheme in reversed(morphemes):
         if morpheme.pos in ("助詞", "特殊", "判定詞"):
             end_index -= 1
         else:
