@@ -1,6 +1,6 @@
 from abc import ABC, abstractmethod
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Optional, Union
+from typing import TYPE_CHECKING
 
 from rhoknp.cohesion.exophora import ExophoraReferent
 from rhoknp.cohesion.predicate import Predicate
@@ -41,7 +41,7 @@ class BaseArgument(ABC):
         self.case: str = case  #: 述語に対する格．
         self.type: ArgumentType = arg_type  #: 項のタイプ．
         self.optional: bool = False  #: 修飾的な項かどうか．
-        self._pas: Optional["Pas"] = None
+        self._pas: "Pas" | None = None
 
     @abstractmethod
     def __str__(self) -> str:
@@ -52,13 +52,12 @@ class BaseArgument(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def __eq__(self, other: Any) -> bool:
+    def __hash__(self) -> int:
         raise NotImplementedError
 
-    @property
-    def is_special(self) -> bool:
-        """外界照応なら True．"""
-        return self.type == ArgumentType.EXOPHORA
+    @abstractmethod
+    def __eq__(self, other: object) -> bool:
+        raise NotImplementedError
 
     @property
     def pas(self) -> "Pas":
@@ -71,6 +70,10 @@ class BaseArgument(ABC):
         """述語項構造．"""
         self._pas = pas
 
+    def is_special(self) -> bool:
+        """外界照応なら True．"""
+        return self.type == ArgumentType.EXOPHORA
+
 
 class EndophoraArgument(BaseArgument):
     """文脈中の基本句に対応する項を表すクラス．
@@ -86,19 +89,22 @@ class EndophoraArgument(BaseArgument):
         case: str,
         base_phrase: "BasePhrase",
         predicate: Predicate,
-        arg_type: Optional[ArgumentType] = None,
+        arg_type: ArgumentType | None = None,
     ) -> None:
         super().__init__(case, arg_type or self._get_arg_type(predicate, base_phrase, case))
         self.base_phrase = base_phrase  #: 項の核となる基本句．
 
     def __repr__(self) -> str:
-        return f"<{self.__module__}.{self.__class__.__name__}: {repr(self.case)}, {repr(self.base_phrase.text)}>"
+        return f"<{self.__module__}.{self.__class__.__name__}: {self.case!r}, {self.base_phrase.text!r}>"
 
     def __str__(self) -> str:
         return self.base_phrase.text
 
-    def __eq__(self, other: Any) -> bool:
-        if isinstance(other, type(self)) is False:
+    def __hash__(self) -> int:
+        return hash((self.case, self.base_phrase, self.pas.predicate if self._pas is not None else None))
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, type(self)):
             return False
         if self._pas is not None and other._pas is not None:
             if self.pas.predicate != other.pas.predicate:
@@ -165,15 +171,18 @@ class ExophoraArgument(BaseArgument):
 
     def __repr__(self) -> str:
         return (
-            f"{self.__class__.__name__}(case={repr(self.case)}, exophora_referent={repr(self.exophora_referent)}, "
-            f"eid={repr(self.eid)})"
+            f"{self.__class__.__name__}(case={self.case!r}, exophora_referent={self.exophora_referent!r}, "
+            f"eid={self.eid!r})"
         )
 
     def __str__(self) -> str:
         return str(self.exophora_referent)
 
-    def __eq__(self, other: Any) -> bool:
-        if isinstance(other, type(self)) is False:
+    def __hash__(self) -> int:
+        return hash((self.case, self.exophora_referent, self.pas.predicate if self._pas is not None else None))
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, type(self)):
             return False
         if self._pas is not None and other._pas is not None:
             if self.pas.predicate != other.pas.predicate:
@@ -181,4 +190,4 @@ class ExophoraArgument(BaseArgument):
         return self.case == other.case and self.exophora_referent == other.exophora_referent
 
 
-Argument = Union[EndophoraArgument, ExophoraArgument]
+Argument = EndophoraArgument | ExophoraArgument
