@@ -1,5 +1,5 @@
 import logging
-from typing import TYPE_CHECKING, Dict, Optional, Set
+from typing import TYPE_CHECKING, ClassVar, Optional
 
 from rhoknp.cohesion.argument import ExophoraArgument
 from rhoknp.cohesion.exophora import ExophoraReferent
@@ -18,16 +18,20 @@ class Entity:
         exophora_referent: 自身が外界照応の照応先に対応するなら照応先の種類. 対応しないなら None.
     """
 
-    def __init__(self, eid: int, exophora_referent: Optional[ExophoraReferent] = None) -> None:
+    def __init__(self, eid: int, exophora_referent: ExophoraReferent | None = None) -> None:
         self.eid = eid  #: エンティティ ID．
         self.exophora_referent = exophora_referent  #: 外界照応の照応先．対応するものがなければ None．
-        self.mentions: Set["BasePhrase"] = set()  #: このエンティティを参照するメンションの集合．
-        self.mentions_nonidentical: Set["BasePhrase"] = set()  #: このエンティティを≒関係で参照するメンションの集合．
+        self.mentions: list["BasePhrase"] = []  #: このエンティティを参照するメンションのリスト．
+        self.mentions_nonidentical: list["BasePhrase"] = []  #: このエンティティを≒関係で参照するメンションのリスト．
 
     @property
-    def mentions_all(self) -> Set["BasePhrase"]:
-        """nonidentical を含めたこのエンティティを参照する全てのメンションの集合．"""
-        return self.mentions | self.mentions_nonidentical
+    def mentions_all(self) -> list["BasePhrase"]:
+        """nonidentical を含めたこのエンティティを参照する全てのメンションのリスト．"""
+        ret = self.mentions.copy()
+        for mention in self.mentions_nonidentical:
+            if mention not in ret:
+                ret.append(mention)
+        return ret
 
     def add_mention(self, mention: "BasePhrase", is_nonidentical: bool = False) -> None:
         """このエンティティを参照するメンションを追加．
@@ -44,12 +48,14 @@ class Entity:
             if mention in self.mentions_all:
                 return
             mention.entities_nonidentical.add(self)
-            self.mentions_nonidentical.add(mention)
+            self.mentions_nonidentical.append(mention)
         else:
             if mention in self.mentions_nonidentical:
                 self.remove_mention(mention)
+            if mention in self.mentions:
+                return
             mention.entities.add(self)
-            self.mentions.add(mention)
+            self.mentions.append(mention)
 
     def remove_mention(self, mention: "BasePhrase") -> None:
         """このエンティティを参照するメンションを削除．
@@ -68,9 +74,9 @@ class Entity:
         if self.exophora_referent:
             return str(self.exophora_referent)
         if self.mentions:
-            return str(list(self.mentions)[0])
+            return next(iter(self.mentions)).text
         elif self.mentions_nonidentical:
-            return str(list(self.mentions_nonidentical)[0])
+            return next(iter(self.mentions_nonidentical)).text
         else:
             return ""
 
@@ -89,12 +95,10 @@ class Entity:
 class EntityManager:
     """文書全体のエンティティを管理．"""
 
-    entities: Dict[int, Entity] = {}  #: ID をキーとするエンティティの辞書．
+    entities: ClassVar[dict[int, Entity]] = {}  #: ID をキーとするエンティティの辞書．
 
     @classmethod
-    def get_or_create_entity(
-        cls, exophora_referent: Optional[ExophoraReferent] = None, eid: Optional[int] = None
-    ) -> Entity:
+    def get_or_create_entity(cls, exophora_referent: ExophoraReferent | None = None, eid: int | None = None) -> Entity:
         """自身が参照するエンティティを作成．
 
         exophora_referent が singleton entity だった場合を除き，新しく Entity のインスタンスを作成して返す．
@@ -106,9 +110,9 @@ class EntityManager:
             eid: エンティティ ID．None の場合自動で割り振る．
 
         Returns:
-             Entity: 作成されたエンティティ．
+            Entity: 作成されたエンティティ．
         """
-        if exophora_referent is not None and exophora_referent.is_singleton is True:
+        if exophora_referent is not None and exophora_referent.is_singleton():
             entities = [e for e in cls.entities.values() if exophora_referent == e.exophora_referent]
             # If a singleton entity already exists, do not create a new entity, but return that entity.
             if entities:
@@ -149,13 +153,13 @@ class EntityManager:
         is_tgt_nonidentical = target_mention is not None and target_entity in target_mention.entities_nonidentical
         is_src_nonidentical = source_entity in source_mention.entities_nonidentical
         if source_entity is target_entity:
-            if is_nonidentical is False:
+            if not is_nonidentical:
                 # When two sides of a triangle formed by source_entity (=target_entity), source_mention, and
                 # target_mention are identical, the other side is also identical.
-                if is_src_nonidentical is False and is_tgt_nonidentical is True:
+                if not is_src_nonidentical and is_tgt_nonidentical:
                     assert target_mention is not None
                     source_entity.add_mention(target_mention, is_nonidentical=False)
-                if is_src_nonidentical is True and is_tgt_nonidentical is False:
+                if is_src_nonidentical and not is_tgt_nonidentical:
                     source_entity.add_mention(source_mention, is_nonidentical=False)
             return
         if target_mention is not None:
@@ -178,7 +182,7 @@ class EntityManager:
             source_entity.add_mention(tm, is_nonidentical=target_entity in tm.entities_nonidentical)
         # Arguments also have entity ids and will be updated.
         source_sentence = source_mention.sentence
-        pas_list = source_mention.document.pas_list if source_sentence.has_document else source_sentence.pas_list
+        pas_list = source_mention.document.pas_list if source_sentence.has_document() else source_sentence.pas_list
         for arg in [arg for pas in pas_list for args in pas.get_all_arguments(relax=False).values() for arg in args]:
             if isinstance(arg, ExophoraArgument) and arg.eid == target_entity.eid:
                 arg.eid = source_entity.eid
