@@ -3,7 +3,7 @@ import logging
 import re
 from collections import defaultdict
 from enum import Enum, auto
-from typing import TYPE_CHECKING, Dict, List, Optional
+from typing import TYPE_CHECKING
 
 from rhoknp.cohesion import EntityManager
 from rhoknp.cohesion.argument import HIRA2KATA, Argument, ArgumentType, EndophoraArgument, ExophoraArgument
@@ -39,8 +39,8 @@ class Pas:
     def __init__(self, predicate: Predicate) -> None:
         self._predicate = predicate
         predicate.pas = self
-        self._arguments: Dict[str, List[Argument]] = defaultdict(list)
-        self.modes: Dict[str, RelMode] = {}
+        self._arguments: dict[str, list[Argument]] = defaultdict(list)
+        self.modes: dict[str, RelMode] = {}
 
     @property
     def predicate(self) -> Predicate:
@@ -48,7 +48,7 @@ class Pas:
         return self._predicate
 
     @property
-    def cases(self) -> List[str]:
+    def cases(self) -> list[str]:
         """属する全ての項の持つ格を集めたリスト．"""
         return [case for case, args in self._arguments.items() if args]
 
@@ -57,7 +57,6 @@ class Pas:
         """文 ID．"""
         return self._predicate.sid
 
-    @property
     def is_empty(self) -> bool:
         """述語が項を持たなければ True．"""
         for args in self._arguments.values():
@@ -65,7 +64,7 @@ class Pas:
                 return False
         return True
 
-    def parse_pas_string(self, base_phrase: "BasePhrase", pas_string: str, format_: CaseInfoFormat):
+    def parse_pas_string(self, base_phrase: "BasePhrase", pas_string: str, format_: CaseInfoFormat) -> None:
         """PAS 文字列から述語項構造を生成する．
 
         Args:
@@ -76,7 +75,9 @@ class Pas:
         # language=RegExp
         cfid_pat = r"(.*?):([^:/]+?)"  # 食べる/たべる:動1
         match = re.match(
-            r"{cfid}(:(?P<args>{args}(;{args})*))?$".format(cfid=cfid_pat, args=self.ARGUMENT_PAT.pattern),
+            r"{cfid}(:(?P<args>{args}(;{args})*))?$".format(  # noqa: UP032
+                cfid=cfid_pat, args=self.ARGUMENT_PAT.pattern
+            ),
             pas_string,
         )
 
@@ -141,7 +142,7 @@ class Pas:
         relax: bool = True,
         include_nonidentical: bool = False,
         include_optional: bool = False,
-    ) -> List[Argument]:
+    ) -> list[Argument]:
         """与えられた格の全ての項を返す．
 
         Args:
@@ -171,7 +172,7 @@ class Pas:
                 elif isinstance(arg, EndophoraArgument):
                     entities = arg.base_phrase.entities_all if include_nonidentical else arg.base_phrase.entities
                 else:
-                    raise AssertionError  # unreachable
+                    raise AssertionError  # noqa: TRY004, unreachable
                 for entity in entities:
                     if entity.exophora_referent is not None:
                         pas.add_argument(ExophoraArgument(case, entity.exophora_referent, entity.eid))
@@ -186,7 +187,7 @@ class Pas:
         relax: bool = True,
         include_nonidentical: bool = False,
         include_optional: bool = False,
-    ) -> Dict[str, List[Argument]]:
+    ) -> dict[str, list[Argument]]:
         """この述語項構造が持つ全ての項を格を key とする辞書形式で返す．
 
         Args:
@@ -194,7 +195,7 @@ class Pas:
             include_nonidentical: True なら nonidentical な項も含めて出力する．
             include_optional: True なら修飾的表現（「すぐに」など）も含めて出力する．
         """
-        all_arguments: Dict[str, List[Argument]] = {}
+        all_arguments: dict[str, list[Argument]] = {}
         for case in self.cases:
             all_arguments[case] = self.get_arguments(
                 case,
@@ -204,7 +205,7 @@ class Pas:
             )
         return all_arguments
 
-    def add_argument(self, argument: Argument, mode: Optional[RelMode] = None) -> None:
+    def add_argument(self, argument: Argument, mode: RelMode | None = None) -> None:
         """述語項構造に項を追加．
 
         Args:
@@ -233,7 +234,7 @@ class Pas:
             logger.info(f"marked {arg} as optional in {self.sid}")
 
     def __repr__(self) -> str:
-        return f"<{self.__module__}.{self.__class__.__name__}: {repr(self.predicate.text)}>"
+        return f"<{self.__module__}.{self.__class__.__name__}: {self.predicate.text!r}>"
 
 
 def normalize_case(case: str) -> str:
