@@ -3,4 +3,4 @@ from rhoknp.processors.knp import KNP
 from rhoknp.processors.kwja import KWJA
 from rhoknp.processors.senter import RegexSenter
 
-__all__ = ["RegexSenter", "Jumanpp", "KNP", "KWJA"]
+__all__ = ["KNP", "KWJA", "Jumanpp", "RegexSenter"]
