@@ -1,19 +1,22 @@
 import re
-from typing import Dict, Optional, Union
 
 
-class SemanticsDict(Dict[str, Union[str, bool]]):
+class SemanticsDict(dict[str, str | bool]):
     """形態素の意味情報を表すクラス．"""
 
     NIL = "NIL"
     PAT = re.compile(rf'(?P<sems>("[^"]+?")|{NIL})')
     SEM_PAT = re.compile(r"(?P<key>[^:\s]+)(:(?P<value>\S+))?(\s|$)")
 
-    def __init__(self, semantics: Optional[Dict[str, Union[str, bool]]] = None, is_nil: bool = False) -> None:
+    def __init__(self, semantics: dict[str, str | bool] | None = None, is_nil: bool = False) -> None:
         if semantics is None:
             semantics = {}
         super().__init__(semantics)
-        self.is_nil: bool = is_nil
+        self.nil: bool = is_nil
+
+    def is_nil(self) -> bool:
+        """NIL なら True．"""
+        return self.nil
 
     @classmethod
     def from_sstring(cls, sstring: str) -> "SemanticsDict":
@@ -34,11 +37,11 @@ class SemanticsDict(Dict[str, Union[str, bool]]):
     def to_sstring(self) -> str:
         """意味情報文字列に変換．"""
         if len(self) == 0:
-            return "" if self.is_nil is False else self.NIL
+            return "" if not self.is_nil() else self.NIL
         return f'"{" ".join(self._item_to_sstring(k, v) for k, v in self.items())}"'
 
     @staticmethod
-    def _item_to_sstring(key: str, value: Union[str, bool]) -> str:
+    def _item_to_sstring(key: str, value: str | bool) -> str:
         if value is False:
             return ""
         if value is True:
