@@ -2,19 +2,20 @@ import dataclasses
 import itertools
 import logging
 import re
-from typing import TYPE_CHECKING, Any, List, Optional, Set
+from functools import cached_property
+from typing import TYPE_CHECKING, Any, Optional
 
 try:
-    from functools import cached_property  # type: ignore
+    from typing import override  # type: ignore[attr-defined]
 except ImportError:
-    from cached_property import cached_property
+    from typing_extensions import override
 
 from rhoknp.cohesion.argument import Argument, EndophoraArgument, ExophoraArgument
 from rhoknp.cohesion.coreference import Entity, EntityManager
 from rhoknp.cohesion.exophora import ExophoraReferent
 from rhoknp.cohesion.pas import CaseInfoFormat, Pas, normalize_case
 from rhoknp.cohesion.predicate import Predicate
-from rhoknp.cohesion.rel import CASE_TYPES, COREF_TYPES, RelMode, RelTag, RelTagList
+from rhoknp.cohesion.rel import RelMode, RelTag, RelTagList
 from rhoknp.props.dependency import DepType
 from rhoknp.props.feature import FeatureDict
 from rhoknp.props.memo import MemoTag
@@ -40,32 +41,51 @@ class BasePhrase(Unit):
 
     def __init__(
         self,
-        parent_index: Optional[int],
-        dep_type: Optional[DepType],
-        features: Optional[FeatureDict] = None,
-        rel_tags: Optional[RelTagList] = None,
-        memo_tag: Optional[MemoTag] = None,
+        parent_index: int | None,
+        dep_type: DepType | None,
+        features: FeatureDict | None = None,
+        rel_tags: RelTagList | None = None,
+        memo_tag: MemoTag | None = None,
     ) -> None:
         super().__init__()
 
         # parent unit
-        self._phrase: Optional["Phrase"] = None
+        self._phrase: "Phrase" | None = None
 
         # child units
-        self._morphemes: Optional[List[Morpheme]] = None
+        self._morphemes: list[Morpheme] | None = None
 
-        self.parent_index: Optional[int] = parent_index  #: 係り先の基本句の文内におけるインデックス．
-        self.dep_type: Optional[DepType] = dep_type  #: 係り受けの種類．
+        self.parent_index: int | None = parent_index  #: 係り先の基本句の文内におけるインデックス．
+        self.dep_type: DepType | None = dep_type  #: 係り受けの種類．
         self.features: FeatureDict = features or FeatureDict()  #: 素性．
         self.rel_tags: RelTagList = rel_tags or RelTagList()  #: 基本句間関係．
         self.memo_tag: MemoTag = memo_tag or MemoTag()  #: タグ付けメモ．
         self.pas: Pas = Pas(Predicate(self))  #: 述語項構造．
-        self.entities: Set[Entity] = set()  #: 参照しているエンティティ．
-        self.entities_nonidentical: Set[Entity] = set()  #: ≒で参照しているエンティティ．
+        self.entities: set[Entity] = set()  #: 参照しているエンティティ．
+        self.entities_nonidentical: set[Entity] = set()  #: ≒で参照しているエンティティ．
 
         self.index = self.count  #: 文内におけるインデックス．
         BasePhrase.count += 1
 
+    def __getstate__(self) -> dict[str, Any]:
+        state = self.__dict__.copy()
+        # Dump a tuple instead of a set so that the __hash__ function won't be called.
+        # `eids` is used to hash uninitialized Entity objects.
+        state["entities"] = tuple(self.entities)
+        state["eids"] = tuple(entity.eid for entity in state["entities"])
+        state["entities_nonidentical"] = tuple(self.entities_nonidentical)
+        state["eids_nonidentical"] = tuple(entity.eid for entity in state["entities_nonidentical"])
+        return state
+
+    def __setstate__(self, state: dict[str, Any]) -> None:
+        # Restore eids to Entity objects for hashing.
+        for entity, eid in zip(state["entities"], state.pop("eids"), strict=True):
+            entity.eid = eid
+        for entity, eid in zip(state["entities_nonidentical"], state.pop("eids_nonidentical"), strict=True):
+            entity.eid = eid
+        self.__dict__.update(state)  # Entity objects are hashed by eid.
+
+    @override
     def __post_init__(self) -> None:
         super().__post_init__()
 
@@ -80,20 +100,24 @@ class BasePhrase(Unit):
             self.pas.parse_pas_string(self, pas_string, format_=CaseInfoFormat.CASE)
 
         # Parse the rel tags.
-        for rel_tag in self.rel_tags:
+        for rel_tag_orig in self.rel_tags:
+            rel_tag = rel_tag_orig
             if rel_tag.sid == "":
                 # The target is considered to be in the same sentence.
                 rel_tag = dataclasses.replace(rel_tag, sid=self.sentence.sid)
-            if rel_tag.type in COREF_TYPES:
+            if rel_tag.is_coreference():
                 if rel_tag.mode not in (RelMode.OR, RelMode.AMBIGUOUS):
                     self._add_coreference(rel_tag)
             else:
-                if rel_tag.type not in CASE_TYPES:
-                    logger.warning(f"{self.sentence.sid}: unknown rel type found: {rel_tag}")
                 self._add_argument(rel_tag)
 
-    def __eq__(self, other: Any) -> bool:
-        if isinstance(other, type(self)) is False:
+    @override
+    def __hash__(self) -> int:
+        return hash((self.parent_unit, self.index))
+
+    @override
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, type(self)):
             return False
         if self.parent_unit != other.parent_unit:
             return False
@@ -102,13 +126,14 @@ class BasePhrase(Unit):
     @cached_property
     def global_index(self) -> int:
         """文書全体におけるインデックス．"""
-        if self.sentence.has_document is False:
+        if not self.sentence.has_document():
             return self.index
-        if self.index > 0:
-            return self.sentence.base_phrases[self.index - 1].global_index + 1
         if self.sentence.index == 0:
             return self.index
-        return self.document.sentences[self.sentence.index - 1].base_phrases[-1].global_index + 1
+        if self.index > 0:
+            return self.sentence.base_phrases[0].global_index + self.index
+        prev_sentence = self.document.sentences[self.sentence.index - 1]
+        return prev_sentence.base_phrases[0].global_index + len(prev_sentence.base_phrases)
 
     @property
     def parent_unit(self) -> Optional["Phrase"]:
@@ -116,7 +141,7 @@ class BasePhrase(Unit):
         return self._phrase
 
     @property
-    def child_units(self) -> Optional[List[Morpheme]]:
+    def child_units(self) -> list[Morpheme] | None:
         """下位の言語単位（形態素）．解析結果にアクセスできないなら None．"""
         return self._morphemes
 
@@ -159,13 +184,13 @@ class BasePhrase(Unit):
         self._phrase = phrase
 
     @property
-    def morphemes(self) -> List[Morpheme]:
+    def morphemes(self) -> list[Morpheme]:
         """形態素のリスト．"""
         assert self._morphemes is not None
         return self._morphemes
 
     @morphemes.setter
-    def morphemes(self, morphemes: List[Morpheme]) -> None:
+    def morphemes(self, morphemes: list[Morpheme]) -> None:
         """形態素のリスト．
 
         Args:
@@ -204,7 +229,7 @@ class BasePhrase(Unit):
         return self.sentence.base_phrases[self.parent_index]
 
     @cached_property
-    def children(self) -> List["BasePhrase"]:
+    def children(self) -> list["BasePhrase"]:
         """この基本句に係っている基本句のリスト．
 
         Raises:
@@ -213,7 +238,7 @@ class BasePhrase(Unit):
         return [base_phrase for base_phrase in self.sentence.base_phrases if base_phrase.parent == self]
 
     @property
-    def entities_all(self) -> Set[Entity]:
+    def entities_all(self) -> set[Entity]:
         """nonidentical も含めた参照している全エンティティの集合．"""
         return self.entities | self.entities_nonidentical
 
@@ -228,14 +253,15 @@ class BasePhrase(Unit):
         match = cls.PAT.match(first_line)
         if match is None:
             raise ValueError(f"malformed base phrase line: {first_line}")
-        parent_index = int(match["pid"]) if match["pid"] is not None else None
-        dep_type = DepType(match["dtype"]) if match["dtype"] is not None else None
-        features = FeatureDict.from_fstring(match["feats"] or "")
-        rel_tags = RelTagList.from_fstring(match["feats"] or "")
-        memo_tag = MemoTag.from_fstring(match["feats"] or "")
-        base_phrase = cls(parent_index, dep_type, features, rel_tags, memo_tag)
+        base_phrase = cls(
+            parent_index=int(match["pid"]) if match["pid"] is not None else None,
+            dep_type=DepType(match["dtype"]) if match["dtype"] is not None else None,
+            features=FeatureDict.from_fstring(match["feats"] or ""),
+            rel_tags=RelTagList.from_fstring(match["feats"] or ""),
+            memo_tag=MemoTag.from_fstring(match["feats"] or ""),
+        )
 
-        morphemes: List[Morpheme] = []
+        morphemes: list[Morpheme] = []
         for line in lines:
             if line.strip() == "":
                 continue
@@ -259,7 +285,7 @@ class BasePhrase(Unit):
         ret += "".join(morpheme.to_knp() for morpheme in self.morphemes)
         return ret
 
-    def get_coreferents(self, include_nonidentical: bool = False, include_self: bool = False) -> Set["BasePhrase"]:
+    def get_coreferents(self, include_nonidentical: bool = False, include_self: bool = False) -> list["BasePhrase"]:
         """この基本句と共参照している基本句の集合を返却．
 
         Args:
@@ -269,14 +295,17 @@ class BasePhrase(Unit):
         Returns:
             共参照している基本句の集合．
         """
-        mentions: Set["BasePhrase"] = set()
-        for entity in self.entities:
-            mentions.update(entity.mentions)
+        mentions: list["BasePhrase"] = [self]
+        for mention in itertools.chain.from_iterable(entity.mentions for entity in self.entities):
+            if mention not in mentions:
+                mentions.append(mention)
         if include_nonidentical is True:
-            for entity in self.entities_nonidentical:
-                mentions.update(entity.mentions)
-        if include_self is False and self in mentions:
-            mentions.remove(self)
+            for mention in itertools.chain.from_iterable(entity.mentions for entity in self.entities_nonidentical):
+                if mention not in mentions:
+                    mentions.append(mention)
+        if include_self is False:
+            while self in mentions:
+                mentions.remove(self)
         return mentions
 
     def _add_argument(self, rel_tag: RelTag) -> None:
@@ -332,7 +361,7 @@ class BasePhrase(Unit):
 
     def _get_target_base_phrase(self, rel_tag: RelTag) -> Optional["BasePhrase"]:
         """rel_tag が指す基本句を返す．見つからなければ None を返す．"""
-        sentences = self.document.sentences if self.sentence.has_document else [self.sentence]
+        sentences = self.document.sentences if self.sentence.has_document() else [self.sentence]
         sentences = [sent for sent in sentences if sent.sid == rel_tag.sid]
         if not sentences:
             logger.warning(f"{self.sentence.sid}: relation with unknown sid found: {rel_tag.sid}")
@@ -349,9 +378,6 @@ class BasePhrase(Unit):
             )
         return target_base_phrase
 
-    def __hash__(self) -> int:
-        return hash((self.global_index, self.sentence.sid))
-
     @staticmethod
     def is_base_phrase_line(line: str) -> bool:
         """基本句行なら True を返す．"""
