@@ -1,10 +1,11 @@
 import logging
+from functools import cached_property
+from typing import TYPE_CHECKING, Optional
 
 try:
-    from functools import cached_property  # type: ignore
+    from typing import override  # type: ignore[attr-defined]
 except ImportError:
-    from cached_property import cached_property
-from typing import TYPE_CHECKING, Any, List, Optional
+    from typing_extensions import override
 
 from rhoknp.cohesion.discourse import DiscourseRelation
 from rhoknp.units.base_phrase import BasePhrase
@@ -28,16 +29,17 @@ class Clause(Unit):
         super().__init__()
 
         # parent unit
-        self._sentence: Optional["Sentence"] = None
+        self._sentence: "Sentence" | None = None
 
         # child units
-        self._phrases: Optional[List[Phrase]] = None
+        self._phrases: list[Phrase] | None = None
 
-        self.discourse_relations: List[DiscourseRelation] = []  #: 談話関係のリスト．
+        self.discourse_relations: list[DiscourseRelation] = []  #: 談話関係のリスト．
 
         self.index = self.count  #: 文内におけるインデックス．
         Clause.count += 1
 
+    @override
     def __post_init__(self) -> None:
         super().__post_init__()
 
@@ -68,8 +70,13 @@ class Clause(Unit):
                     if relation not in relation.modifier.discourse_relations:
                         relation.modifier.discourse_relations.append(relation)
 
-    def __eq__(self, other: Any) -> bool:
-        if isinstance(other, type(self)) is False:
+    @override
+    def __hash__(self) -> int:
+        return hash((self.parent_unit, self.index))
+
+    @override
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, type(self)):
             return False
         if self.parent_unit != other.parent_unit:
             return False
@@ -78,13 +85,14 @@ class Clause(Unit):
     @cached_property
     def global_index(self) -> int:
         """文書全体におけるインデックス．"""
-        if self.sentence.has_document is False:
+        if not self.sentence.has_document():
             return self.index
-        if self.index > 0:
-            return self.sentence.clauses[self.index - 1].global_index + 1
         if self.sentence.index == 0:
             return self.index
-        return self.document.sentences[self.sentence.index - 1].clauses[-1].global_index + 1
+        if self.index > 0:
+            return self.sentence.clauses[0].global_index + self.index
+        prev_sentence = self.document.sentences[self.sentence.index - 1]
+        return prev_sentence.clauses[0].global_index + len(prev_sentence.clauses)
 
     @property
     def parent_unit(self) -> Optional["Sentence"]:
@@ -92,7 +100,7 @@ class Clause(Unit):
         return self._sentence
 
     @property
-    def child_units(self) -> Optional[List[Phrase]]:
+    def child_units(self) -> list[Phrase] | None:
         """下位の言語単位（文節）．解析結果にアクセスできないなら None．"""
         return self._phrases
 
@@ -121,13 +129,13 @@ class Clause(Unit):
         self._sentence = sentence
 
     @property
-    def phrases(self) -> List[Phrase]:
+    def phrases(self) -> list[Phrase]:
         """文節のリスト．"""
         assert self._phrases is not None
         return self._phrases
 
     @phrases.setter
-    def phrases(self, phrases: List[Phrase]) -> None:
+    def phrases(self, phrases: list[Phrase]) -> None:
         """文節のリスト．
 
         Args:
@@ -138,19 +146,19 @@ class Clause(Unit):
         self._phrases = phrases
 
     @property
-    def base_phrases(self) -> List[BasePhrase]:
+    def base_phrases(self) -> list[BasePhrase]:
         """基本句のリスト．"""
         return [base_phrase for phrase in self.phrases for base_phrase in phrase.base_phrases]
 
     @property
-    def morphemes(self) -> List[Morpheme]:
+    def morphemes(self) -> list[Morpheme]:
         """形態素のリスト．"""
         return [morpheme for base_phrase in self.base_phrases for morpheme in base_phrase.morphemes]
 
     @cached_property
     def head(self) -> BasePhrase:
         """節主辞の基本句．"""
-        heads: List[BasePhrase] = []
+        heads: list[BasePhrase] = []
         for base_phrase in self.base_phrases:
             if "節-主辞" in base_phrase.features:
                 heads.append(base_phrase)
@@ -180,16 +188,14 @@ class Clause(Unit):
         return None
 
     @cached_property
-    def children(self) -> List["Clause"]:
+    def children(self) -> list["Clause"]:
         """この節に係っている節のリスト．"""
         return [clause for clause in self.sentence.clauses if clause.parent == self]
 
-    @cached_property
     def is_adnominal(self) -> bool:
         """連体修飾節なら True．"""
         return self.end.features.get("節-区切", "") == "連体修飾"
 
-    @cached_property
     def is_sentential_complement(self) -> bool:
         """補文節なら True．"""
         return self.end.features.get("節-区切", "") == "補文"
@@ -203,7 +209,7 @@ class Clause(Unit):
         """
         clause = cls()
         phrases = []
-        phrase_lines: List[str] = []
+        phrase_lines: list[str] = []
         for line in knp_text.split("\n"):
             if not line.strip():
                 continue
@@ -211,9 +217,8 @@ class Clause(Unit):
                 phrases.append(Phrase.from_knp("\n".join(phrase_lines)))
                 phrase_lines = []
             phrase_lines.append(line)
-        else:
-            phrase = Phrase.from_knp("\n".join(phrase_lines))
-            phrases.append(phrase)
+        phrase = Phrase.from_knp("\n".join(phrase_lines))
+        phrases.append(phrase)
         clause.phrases = phrases
         return clause
 
