@@ -1,6 +1,11 @@
 import logging
 import re
-from typing import TYPE_CHECKING, Any, List, Optional, Union
+from typing import TYPE_CHECKING, Optional
+
+try:
+    from typing import override  # type: ignore[attr-defined]
+except ImportError:
+    from typing_extensions import override
 
 from rhoknp.cohesion import EntityManager, Pas
 from rhoknp.props.named_entity import NamedEntity
@@ -9,7 +14,7 @@ from rhoknp.units.clause import Clause
 from rhoknp.units.morpheme import Morpheme
 from rhoknp.units.phrase import Phrase
 from rhoknp.units.unit import Unit
-from rhoknp.utils.util import _extract_did_and_sid
+from rhoknp.utils.comment import extract_did_and_sid, is_comment_line
 
 if TYPE_CHECKING:
     from rhoknp.units.document import Document
@@ -30,7 +35,7 @@ class Sentence(Unit):
     SID_PAT_WAC = re.compile(r"^(?P<sid>(?P<did>wiki\d{8})(-\d{2})(-\d{2})?)$")
     count = 0
 
-    def __init__(self, text: Optional[str] = None):
+    def __init__(self, text: str | None = None) -> None:
         super().__init__()
         if text is not None:
             self.text = text.replace("\r", "").replace("\n", "")
@@ -42,39 +47,47 @@ class Sentence(Unit):
         EntityManager.reset()
 
         # parent unit
-        self._document: Optional["Document"] = None
+        self._document: "Document" | None = None
 
         # child units
-        self._clauses: Optional[List[Clause]] = None
-        self._phrases: Optional[List[Phrase]] = None
-        self._morphemes: Optional[List[Morpheme]] = None
+        self._clauses: list[Clause] | None = None
+        self._phrases: list[Phrase] | None = None
+        self._morphemes: list[Morpheme] | None = None
 
         self.sent_id: str = ""
         self.doc_id: str = ""
         self.misc_comment: str = ""
 
-        self.named_entities: List[NamedEntity] = []
+        self.named_entities: list[NamedEntity] = []
 
         self.index = self.count  #: 文書全体におけるインデックス．
         Sentence.count += 1
 
+    @override
     def __post_init__(self) -> None:
         super().__post_init__()
 
         # Find named entities in the sentence.
         self.named_entities = []
-        if self.need_knp is False:
+        if not self.is_knp_required():
             for base_phrase in self.base_phrases:
                 if "NE" not in base_phrase.features:
                     continue
-                fstring = f'<NE:{base_phrase.features["NE"]}>'
+                assert isinstance(base_phrase.features["NE"], str)
+                ne_value = base_phrase.features["NE"].replace(">", r"\>")
+                fstring = f"<NE:{ne_value}>"
                 candidate_morphemes = self.morphemes[: base_phrase.morphemes[-1].index + 1]
                 named_entity = NamedEntity.from_fstring(fstring, candidate_morphemes)
                 if named_entity is not None:
                     self.named_entities.append(named_entity)
 
-    def __eq__(self, other: Any) -> bool:
-        if isinstance(other, type(self)) is False:
+    @override
+    def __hash__(self) -> int:
+        return hash((self.sent_id, self.text))
+
+    @override
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, type(self)):
             return False
         return self.sent_id == other.sent_id and self.text == other.text
 
@@ -89,7 +102,7 @@ class Sentence(Unit):
         return self._document
 
     @property
-    def child_units(self) -> Optional[Union[List[Clause], List[Phrase], List[Morpheme]]]:
+    def child_units(self) -> list[Clause] | list[Phrase] | list[Morpheme] | None:
         """下位の言語単位（節もしくは形態素）のリスト．解析結果にアクセスできないなら None．
 
         .. note::
@@ -153,7 +166,7 @@ class Sentence(Unit):
         self._document = document
 
     @property
-    def clauses(self) -> List[Clause]:
+    def clauses(self) -> list[Clause]:
         """節のリスト．
 
         Raises:
@@ -164,7 +177,7 @@ class Sentence(Unit):
         return self._clauses
 
     @clauses.setter
-    def clauses(self, clauses: List[Clause]) -> None:
+    def clauses(self, clauses: list[Clause]) -> None:
         """節のリスト．
 
         Args:
@@ -175,7 +188,7 @@ class Sentence(Unit):
         self._clauses = clauses
 
     @property
-    def phrases(self) -> List[Phrase]:
+    def phrases(self) -> list[Phrase]:
         """文節のリスト．
 
         Raises:
@@ -188,7 +201,7 @@ class Sentence(Unit):
         raise AttributeError("phrases have not been set")
 
     @phrases.setter
-    def phrases(self, phrases: List[Phrase]) -> None:
+    def phrases(self, phrases: list[Phrase]) -> None:
         """文節のリスト．
 
         Args:
@@ -199,7 +212,7 @@ class Sentence(Unit):
         self._phrases = phrases
 
     @property
-    def base_phrases(self) -> List[BasePhrase]:
+    def base_phrases(self) -> list[BasePhrase]:
         """基本句のリスト．
 
         Raises:
@@ -208,7 +221,7 @@ class Sentence(Unit):
         return [base_phrase for phrase in self.phrases for base_phrase in phrase.base_phrases]
 
     @property
-    def morphemes(self) -> List[Morpheme]:
+    def morphemes(self) -> list[Morpheme]:
         """形態素のリスト．
 
         Raises:
@@ -223,7 +236,7 @@ class Sentence(Unit):
         raise AttributeError("morphemes have not been set")
 
     @morphemes.setter
-    def morphemes(self, morphemes: List[Morpheme]) -> None:
+    def morphemes(self, morphemes: list[Morpheme]) -> None:
         """形態素のリスト．
 
         Args:
@@ -252,7 +265,7 @@ class Sentence(Unit):
         Args:
             comment: コメント行．
         """
-        doc_id, sent_id, rest = _extract_did_and_sid(
+        doc_id, sent_id, rest = extract_did_and_sid(
             comment, patterns=[self.SID_PAT_KWDLC, self.SID_PAT_WAC, self.SID_PAT]
         )
         if sent_id is not None:
@@ -262,33 +275,13 @@ class Sentence(Unit):
         self.misc_comment = rest
 
     @property
-    def pas_list(self) -> List[Pas]:
+    def pas_list(self) -> list[Pas]:
         """述語項構造のリスト．
 
         Raises:
             AttributeError: 解析結果にアクセスできない場合．
         """
-        return [base_phrase.pas for base_phrase in self.base_phrases if base_phrase.pas.is_empty is False]
-
-    @property
-    def has_document(self) -> bool:
-        """文書が設定されていたら True．"""
-        return self._document is not None
-
-    @property
-    def need_jumanpp(self) -> bool:
-        """Juman++ による形態素解析がまだなら True．"""
-        return self._morphemes is None and self._phrases is None and self._clauses is None
-
-    @property
-    def need_knp(self) -> bool:
-        """KNP による構文解析がまだなら True．"""
-        return self._phrases is None and self._clauses is None
-
-    @property
-    def need_clause_tag(self) -> bool:
-        """KNP による節-主辞・節-区切のタグ付与がまだなら True．"""
-        return self._clauses is None
+        return [base_phrase.pas for base_phrase in self.base_phrases if not base_phrase.pas.is_empty()]
 
     @classmethod
     def from_raw_text(cls, text: str, post_init: bool = True) -> "Sentence":
@@ -307,7 +300,7 @@ class Sentence(Unit):
         for line in text.split("\n"):
             if line.strip() == "":
                 continue
-            if cls.is_comment_line(line):
+            if is_comment_line(line):
                 sentence.comment = line
             else:
                 sentence.text += line.replace("\r", "")
@@ -342,12 +335,12 @@ class Sentence(Unit):
             >>> sent = Sentence.from_jumanpp(jumanpp_text)
         """
         sentence = cls()
-        morphemes: List[Morpheme] = []
-        jumanpp_lines: List[str] = []
+        morphemes: list[Morpheme] = []
+        jumanpp_lines: list[str] = []
         for line in jumanpp_text.split("\n"):
             if line.strip() == "":
                 continue
-            if cls.is_comment_line(line):
+            if is_comment_line(line):
                 sentence.comment = line
                 continue
             if Morpheme.is_morpheme_line(line):
@@ -360,10 +353,13 @@ class Sentence(Unit):
                 jumanpp_lines.append(line)
                 continue
             if line.strip() == cls.EOS:
-                if jumanpp_lines:
-                    morphemes.append(Morpheme.from_jumanpp("\n".join(jumanpp_lines)))
                 break
             raise ValueError(f"malformed line: {line}")
+        else:
+            logger.warning(f"sentence does not end with EOS: {jumanpp_lines}")
+
+        if jumanpp_lines:
+            morphemes.append(Morpheme.from_jumanpp("\n".join(jumanpp_lines)))
         sentence.morphemes = morphemes
         if post_init is True:
             sentence.__post_init__()
@@ -404,14 +400,14 @@ class Sentence(Unit):
         lines = knp_text.split("\n")
         sentence = cls()
         has_clause_boundary = any("節-区切" in line for line in lines if BasePhrase.is_base_phrase_line(line))
-        clauses: List[Clause] = []
-        phrases: List[Phrase] = []
-        child_lines: List[str] = []
+        clauses: list[Clause] = []
+        phrases: list[Phrase] = []
+        child_lines: list[str] = []
         is_clause_end = False
         for line in lines:
             if line.strip() == "":
                 continue
-            if cls.is_comment_line(line):
+            if is_comment_line(line):
                 sentence.comment = line
                 continue
             if Phrase.is_phrase_line(line):
@@ -433,13 +429,17 @@ class Sentence(Unit):
                 child_lines.append(line)
                 continue
             if line.strip() == cls.EOS:
-                if has_clause_boundary:
-                    clauses.append(Clause.from_knp("\n".join(child_lines)))
-                else:
-                    phrases.append(Phrase.from_knp("\n".join(child_lines)))
                 break
             raise ValueError(f"malformed line: {line}")
-        if has_clause_boundary is True:
+        else:
+            logger.warning(f"sentence does not end with EOS: {child_lines}")
+
+        if child_lines:
+            if has_clause_boundary:
+                clauses.append(Clause.from_knp("\n".join(child_lines)))
+            else:
+                phrases.append(Phrase.from_knp("\n".join(child_lines)))
+        if has_clause_boundary:
             sentence.clauses = clauses
         else:
             sentence.phrases = phrases
@@ -447,6 +447,22 @@ class Sentence(Unit):
             sentence.__post_init__()
         return sentence
 
+    def has_document(self) -> bool:
+        """文書が設定されていたら True．"""
+        return self._document is not None
+
+    def is_jumanpp_required(self) -> bool:
+        """Juman++ による形態素解析がまだなら True．"""
+        return self._morphemes is None and self._phrases is None and self._clauses is None
+
+    def is_knp_required(self) -> bool:
+        """KNP による構文解析がまだなら True．"""
+        return self._phrases is None and self._clauses is None
+
+    def is_clause_tag_required(self) -> bool:
+        """KNP による節-主辞・節-区切のタグ付与がまだなら True．"""
+        return self._clauses is None
+
     def to_raw_text(self) -> str:
         """生テキストフォーマットに変換．"""
         ret = ""
@@ -486,20 +502,8 @@ class Sentence(Unit):
         .. note::
             解析結果に対する編集を有効にする際に実行する必要がある．
         """
-        if self.need_knp is False:
+        if not self.is_knp_required():
             return Sentence.from_knp(self.to_knp())
-        elif self.need_jumanpp is False:
+        if not self.is_jumanpp_required():
             return Sentence.from_jumanpp(self.to_jumanpp())
         return Sentence.from_raw_text(self.to_raw_text())
-
-    @staticmethod
-    def is_comment_line(line: str) -> bool:
-        """コメント行なら True を返す．
-
-        Args:
-            line: 解析結果の一行．
-
-        .. note::
-            JUMAN/KNP では # から始まる行がコメントとみなされる．
-        """
-        return line.startswith("#") and not Morpheme.is_morpheme_line(line)
