@@ -1,12 +1,13 @@
 from abc import ABC, abstractmethod
-from typing import Any, Optional, Sequence
+from collections.abc import Sequence
+from typing import Optional
 
 
 class Unit(ABC):
     """言語単位の基底クラス・"""
 
     def __init__(self) -> None:
-        self._text: Optional[str] = None
+        self._text: str | None = None
 
     def __post_init__(self) -> None:
         if self.child_units is not None:
@@ -14,14 +15,18 @@ class Unit(ABC):
                 child_unit.__post_init__()
 
     @abstractmethod
-    def __eq__(self, other: Any) -> bool:
+    def __hash__(self) -> int:
+        raise NotImplementedError
+
+    @abstractmethod
+    def __eq__(self, other: object) -> bool:
         raise NotImplementedError
 
     def __str__(self) -> str:
         return self.text
 
     def __repr__(self) -> str:
-        return f"<{self.__module__}.{self.__class__.__name__}: {repr(self.text)}>"
+        return f"<{self.__module__}.{self.__class__.__name__}: {self.text!r}>"
 
     @property
     @abstractmethod
@@ -31,7 +36,7 @@ class Unit(ABC):
 
     @property
     @abstractmethod
-    def child_units(self) -> Optional[Sequence["Unit"]]:
+    def child_units(self) -> Sequence["Unit"] | None:
         """下位の言語単位．"""
         raise NotImplementedError
 
